@@ -1,14 +1,18 @@
 # -*- coding: utf-8 -*-
+import os
+import io
 import json
 import sys
 import shutil
 from decimal import Decimal
 from django.db import models
+from django.conf import settings
 from django.core.files.base import ContentFile
 from djangoplus.utils.formatter import to_ascii
 from djangoplus.ui.components.forms import fields as form_fields
 from django.db.models.fields.files import ImageFieldFile
 from djangoplus.utils.metadata import get_metadata, getattr2
+from djangoplus.utils.storage import dropbox
 
 
 # Superclass #
@@ -37,12 +41,17 @@ class CharField(models.CharField, FieldPlus):
     def __init__(self, *args, **kwargs):
         max_length = kwargs.pop('max_length', 255)
         choices = kwargs.get('choices')
+        self.mask = kwargs.pop('mask', None)
         if choices and type(choices[0]) not in (list, tuple):
             kwargs.update(choices=[[choice, choice] for choice in choices])
         super(CharField, self).__init__(*args, max_length=max_length, **kwargs)
 
     def formfield(self, **kwargs):
-        kwargs.setdefault('form_class', form_fields.CharField)
+        if self.mask:
+            kwargs.setdefault('form_class', form_fields.MaskField)
+            kwargs.setdefault('mask', self.mask)
+        else:
+            kwargs.setdefault('form_class', form_fields.CharField)
         kwargs.setdefault('choices_form_class', form_fields.TypedChoiceField)
         field = super(CharField, self).formfield(**kwargs)
         return field
@@ -197,7 +206,10 @@ class NullBooleanField(models.NullBooleanField, FieldPlus):
 class OneToOneField(models.OneToOneField, FieldPlus):
 
     def __init__(self, *args, **kwargs):
-        kwargs.update(on_delete=models.SET_NULL)
+        if kwargs.get('null'):
+            kwargs.update(on_delete=models.SET_NULL)
+        else:
+            kwargs.update(on_delete=models.CASCADE)
         super(OneToOneField, self).__init__(*args, **kwargs)
 
     def formfield(self, **kwargs):
@@ -270,6 +282,7 @@ class ManyToManyField(models.ManyToManyField, FieldPlus):
             field.queryset = field.queryset.filter(**self.queryset_filter)
         return field
 
+
 MultipleModelChoiceField = ManyToManyField
 
 
@@ -287,9 +300,27 @@ class OneToManyField(ManyToManyField):
         form_field.one_to_many_max = self.one_to_many_max
         return form_field
 
+# Icon Field #
+
+
+class IconField(CharField):
+    def formfield(self, **kwargs):
+        kwargs.setdefault('form_class', form_fields.IconChoiceField)
+        return super(IconField, self).formfield(**kwargs)
+
+
 # File Fields #
 
+
 class FileField(models.FileField, FieldPlus):
+
+    def __init__(self, *args, **kwargs):
+        upload_to = kwargs.get('upload_to', None)
+        if upload_to and type(upload_to) == str and 'dropbox:' in upload_to:
+            kwargs['storage'] = dropbox.DropboxStorage()
+            kwargs['upload_to'] = upload_to[8:]
+        super(FileField, self).__init__(*args, **kwargs)
+
     def formfield(self, **kwargs):
         kwargs.setdefault('form_class', form_fields.FileField)
         return super(FileField, self).formfield(**kwargs)
@@ -324,9 +355,10 @@ class ImageFieldFile(ImageFieldFile):
                     setattr(self, 'large', get_size(self, size))
 
     def generate_thumb(self, img, thumb_size, file_format):
-        # return img
-        import io
-        from PIL import Image
+        try:
+            from PIL import Image
+        except ImportError:
+            return img
         img.seek(0)  # see http://code.djangoproject.com/ticket/8222 for details
         image = Image.open(img)
 
@@ -356,13 +388,13 @@ class ImageFieldFile(ImageFieldFile):
             image2 = image
             image2.thumbnail(thumb_size, Image.ANTIALIAS)
 
-        io = io.StringIO()
+        tmp = io.BytesIO()
         # PNG and GIF are the same, JPG is JPEG
         if file_format.upper() == 'JPG':
             file_format = 'JPEG'
 
-        image2.save(io, file_format)
-        return ContentFile(io.getvalue())
+        image2.save(tmp, file_format)
+        return ContentFile(tmp.getvalue())
 
     def save(self, name, content, save=True):
         super(ImageFieldFile, self).save(name, content, save)
@@ -409,9 +441,27 @@ class ImageField(models.ImageField, FieldPlus):
         self.width_field = width_field
         self.height_field = height_field
         self.sizes = sizes
-        default = '/static/images/blank.png'
-        if 'default' in kwargs:
-            default = kwargs.pop('default')
+        default = kwargs.pop('default', None) #  '/static/images/blank.png'
+        if default:
+            if type(default) == bytes:
+                default = default.decode()
+            if default.startswith('/static'):
+                image_name = default.split('/')[-1]
+                media_image_path = os.path.join(settings.MEDIA_ROOT, image_name)
+                if not os.path.exists(media_image_path):
+                    if not os.path.exists(settings.MEDIA_ROOT):
+                        os.mkdir(settings.MEDIA_ROOT)
+                    for app_label in settings.INSTALLED_APPS:
+                        app_dir = os.path.dirname(__import__(app_label, fromlist=app_label.split('.')).__file__)
+                        static_image_path = os.path.join(app_dir, default[1:])
+                        if os.path.exists(static_image_path):
+                            if not os.path.exists(media_image_path):
+                                shutil.copy(static_image_path, media_image_path)
+                default = image_name
+        upload_to = kwargs.get('upload_to', None)
+        if upload_to and type(upload_to) == str and 'dropbox:' in upload_to:
+            kwargs['storage'] = dropbox.DropboxStorage()
+            kwargs['upload_to'] = upload_to[8:]
         super(ImageField, self).__init__(default=default, verbose_name=self.verbose_name, **kwargs)
 
     def formfield(self, **kwargs):
@@ -462,6 +512,15 @@ class CarPlateField(CharField, FieldPlus):
         return super(CarPlateField, self).formfield(**kwargs)
 
 
+class MercosulCarPlateField(CharField, FieldPlus):
+    def __init__(self, **kwargs):
+        super(MercosulCarPlateField, self).__init__(**kwargs)
+
+    def formfield(self, **kwargs):
+        kwargs.setdefault('form_class', form_fields.MercosulCarPlateField)
+        return super(MercosulCarPlateField, self).formfield(**kwargs)
+
+
 class PhoneField(CharField, FieldPlus):
     def formfield(self, **kwargs):
         kwargs.setdefault('form_class', form_fields.PhoneField)
@@ -504,6 +563,12 @@ class Decimal3(Decimal):
         return Decimal.__new__(cls, value=value or "0", context=context)
 
 
+class Decimal1(Decimal):
+    def __new__(cls, value="0", context=None):
+        cls.decimal1 = True
+        return Decimal.__new__(cls, value=value or "0", context=context)
+
+
 class DecimalField3(models.DecimalField, FieldPlus):
     def __init__(self, *args, **kwargs):
         decimal_places = kwargs.pop('decimal_places', 3)
@@ -519,6 +584,21 @@ class DecimalField3(models.DecimalField, FieldPlus):
         return value
 
 
+class DecimalField1(models.DecimalField, FieldPlus):
+    def __init__(self, *args, **kwargs):
+        decimal_places = kwargs.pop('decimal_places', 1)
+        max_digits = kwargs.pop('max_digits', 9)
+        super(DecimalField1, self).__init__(*args, decimal_places=decimal_places, max_digits=max_digits, **kwargs)
+
+    def formfield(self, **kwargs):
+        kwargs.setdefault('form_class', form_fields.DecimalField1)
+        return super(DecimalField1, self).formfield(**kwargs)
+
+    def from_db_value(self, value, expression, connection, context):
+        value = Decimal1(value)
+        return value
+
+
 class CreditCardField(CharField, FieldPlus):
     def formfield(self, **kwargs):
         kwargs.setdefault('form_class', form_fields.CreditCardField)
@@ -565,3 +645,13 @@ class TreeIndexField(CharField):
             kwargs.update(display=None)
         kwargs.update(blank=True, exclude=True, null=True)
         super(TreeIndexField, self).__init__(*args, **kwargs)
+
+
+class ColorField(CharField):
+    def __init__(self, *args, **kwargs):
+        super(ColorField, self).__init__(*args, **kwargs)
+        self.formatter = 'color'
+
+    def formfield(self, **kwargs):
+        kwargs.setdefault('form_class', form_fields.ColorField)
+        return super(ColorField, self).formfield(**kwargs)
