@@ -1,21 +1,22 @@
 # -*- coding: utf-8 -*-
 import copy
-import requests
 import json
+import urllib
 from django.conf import settings
 from django.db import transaction
 from django.template import loader
-from djangoplus.ui.components.forms.fields import *
-from djangoplus.ui.components.forms import widgets
 from django import forms as django_forms
 from django.forms.forms import BoundField
 from djangoplus.templatetags import mobile
 from django.utils.html import conditional_escape
-from djangoplus.utils.metadata import get_metadata, iterable
 from django.db.models.fields import NOT_PROVIDED
+from django.utils.translation import ugettext as _
+from djangoplus.ui.components.forms import widgets
+from djangoplus.ui.components.forms.fields import *
+from djangoplus.utils.metadata import get_metadata, iterable
+
+
 ValidationError = django_forms.ValidationError
-DEFAULT_FORM_TITLE = 'Formulário'
-DEFAULT_SUBMIT_LABEL = 'Enviar'
 
 
 class Form(django_forms.Form):
@@ -25,7 +26,8 @@ class Form(django_forms.Form):
     def __init__(self, request, *args, **kwargs):
         metaclass = hasattr(self.__class__, 'Meta') and self.__class__.Meta or None
         self.request = request
-        self.method = kwargs.pop('method', None) or metaclass and hasattr(metaclass, 'method') and metaclass.method or 'post'
+        self.method = kwargs.pop('method', None) or metaclass and hasattr(
+            metaclass, 'method') and metaclass.method or 'post'
         self.horizontal = True
         self.id = self.__class__.__name__.lower()
         self.inline = kwargs.pop('inline', False)
@@ -34,10 +36,11 @@ class Form(django_forms.Form):
         self.str_hidden = ''
         self.inner_forms = []
         self.configured_fieldsets = []
-        self.submit_label = DEFAULT_SUBMIT_LABEL
-        self.title = DEFAULT_FORM_TITLE
+        self.submit_label = _('Send')
+        self.title = _('Form')
         self.is_inner = False
         self.captcha = False
+        self.component = kwargs.pop('component', None)
 
         if self.method.lower() == 'post':
             kwargs['data'] = request.POST or None
@@ -61,15 +64,18 @@ class Form(django_forms.Form):
             self.note = hasattr(metaclass, 'note') and metaclass.note or ''
             self.is_inner = hasattr(metaclass, 'is_inner') and metaclass.is_inner or False
             self.horizontal = hasattr(metaclass, 'horizontal') and metaclass.horizontal or False
-            self.perm_or_group = hasattr(metaclass, 'perm_or_group') and iterable(metaclass.perm_or_group) or self.perm_or_group
+            self.perm_or_group = hasattr(metaclass, 'perm_or_group') and iterable(
+                metaclass.perm_or_group) or self.perm_or_group
             self.captcha = hasattr(metaclass, 'captcha') and metaclass.captcha or False
 
             if hasattr(metaclass, 'submit_label'):
                 self.submit_label = metaclass.submit_label
             elif hasattr(self, 'instance'):
-                self.submit_label = self.instance.pk and 'Atualizar' or 'Cadastrar'
+                self.submit_label = _('Save')
 
             self.submit_style = hasattr(metaclass, 'submit_style') and metaclass.submit_style or 'default'
+            self.cancel_style = hasattr(metaclass, 'cancel_style') and metaclass.cancel_style or 'default'
+            self.cancel_button = not hasattr(metaclass, 'cancel_button') and True or metaclass.cancel_button
             self.method = hasattr(metaclass, 'method') and metaclass.method or 'post'
 
         for field_name in self.fields:
@@ -99,16 +105,21 @@ class Form(django_forms.Form):
                         if role_username and self.request.user.groups.filter(name=subclass._meta.verbose_name):
                             obj = subclass.objects.get(**{role_username: self.request.user.username})
                     if obj:
-                        groups = self.request.user.find_groups(self.perm_or_group, get_metadata(obj.__class__, 'verbose_name'))
-                        # if the user is not superuser or there is only one group that allows the user to register the object
+                        groups = self.request.user.find_groups(self.perm_or_group, get_metadata(
+                            obj.__class__, 'verbose_name'))
+                        # if the user is not superuser or there is only one group that allows the user to register
+                        # the object
                         if not self.request.user.is_superuser and not groups.exists():
                             field.widget = widgets.HiddenInput(attrs={'value': obj.pk})
                             # field.widget = widgets.DisplayInput(obj)
                             continue
 
-                if self.request.user.is_authenticated and (not hasattr(field, 'ignore_lookup') or not field.ignore_lookup):
+                ignore_lookup = hasattr(field, 'ignore_lookup') and field.ignore_lookup or False
+                if self.request.user.is_authenticated and not ignore_lookup:
                     if not self.is_inner:
-                        field.queryset = field.queryset.all(self.request.user, obj=hasattr(self, 'instance') and self.instance or None)
+                        field.queryset = field.queryset.contextualize(
+                            self.request.user, obj=hasattr(self, 'instance') and self.instance or None
+                        )
 
                 if True:#hasattr(field.queryset.model._meta, 'organization_lookup') or hasattr(field.queryset.model,'organization_ptr'):
                     from djangoplus.admin.models import Organization
@@ -123,7 +134,8 @@ class Form(django_forms.Form):
                     if issubclass(field.queryset.model, Unit):
                         if not isinstance(field, MultipleModelChoiceField) and field.queryset.count() == 1:
                             obj = field.queryset[0]
-                            field.widget = widgets.DisplayInput(obj)
+                            field.widget = widgets.HiddenInput(attrs={'value': obj.pk})
+                            # field.widget = widgets.DisplayInput(obj)
 
                 if hasattr(field.widget, 'lazy') and mobile(self.request):
                     field.widget.lazy = False
@@ -160,23 +172,31 @@ class Form(django_forms.Form):
         for title, fieldset in self.fieldsets:
             field_names = fieldset.get('fields', ())
             relation_names = fieldset.get('relations', ())
-            for name_or_tuple in tuple(field_names) + tuple(relation_names):
+            image_field_name = fieldset.get('image')
+            image_field_names = ()
+            if image_field_name:
+                image_field_names = image_field_name,
+            for name_or_tuple in tuple(field_names) + tuple(relation_names) + image_field_names:
                 for name in iterable(name_or_tuple):
                     fieldset_field_names.append(name)
         for field_name in list(self.fields.keys()):
             if field_name not in fieldset_field_names:
                 extra_fieldset_field_names.append(field_name)
         if extra_fieldset_field_names:
-            self.fieldsets += ('Outros', {'fields': extra_fieldset_field_names, }),
+            self.fieldsets += ('Outros Dados', {'fields': extra_fieldset_field_names, }),
 
         for title, fieldset in self.fieldsets:
             title = '::' in title and title.split('::')[1] or title.split('::')[0]
             field_names = fieldset.get('fields', ())
             relation_names = fieldset.get('relations', ())
+            image_field_name = fieldset.get('image')
+            image_field_names = ()
+            if image_field_name:
+                image_field_names = image_field_name,
 
             configured_fieldset = dict(title=title, tuples=[], one_to_one=[], one_to_many=[])
 
-            for name_or_tuple in tuple(field_names) + tuple(relation_names):
+            for name_or_tuple in tuple(field_names) + tuple(relation_names) + image_field_names:
                 fields = []
                 for name in iterable(name_or_tuple):
                     if name in self.fields:
@@ -204,6 +224,8 @@ class Form(django_forms.Form):
 
                     elif name in one_to_one_fields:
                         field = one_to_one_fields[name]
+                        if not hasattr(self.instance, '{}_id'.format(name)):
+                            continue
                         one_to_one_id = getattr(self.instance, '{}_id'.format(name))
                         form = factory.get_one_to_one_form(self.request, self.instance, name, one_to_one_id,
                                                            partial=True, prefix=name)
@@ -266,12 +288,14 @@ class Form(django_forms.Form):
             captcha_url = 'https://www.google.com/recaptcha/api/siteverify'
             captcha_secret = settings.CAPTCHA_SECRET
             if captcha_response:
-                data = dict(secret=captcha_secret, response=captcha_response)
-                response = json.loads(requests.post(captcha_url, data).content.decode('utf-8'))
-                if not response.get('success'):
-                    raise ValidationError('Confirme que você não é um robô.')
+                if not self.is_inner:
+                    data = dict(secret=captcha_secret, response=captcha_response)
+                    response = json.loads(urllib.request.urlopen(
+                        captcha_url, urllib.parse.urlencode(data).encode('utf-8')).read().decode('utf-8'))
+                    if not response.get('success'):
+                        raise ValidationError(_('Confirm that you are not a robot.'))
             else:
-                raise ValidationError('Confirme que você não é um robô.')
+                raise ValidationError(_('Confirm that you are not a robot.'))
         return super(Form, self).clean(*args, **kwargs)
 
     def is_valid(self, *args, **kwargs):
@@ -295,7 +319,7 @@ class Form(django_forms.Form):
             for field_name in self.fields:
                 self.fields[field_name].widget.attrs['placeholder'] = self.fields[field_name].label
                 self.fields[field_name].widget.attrs['data-placeholder'] = self.fields[field_name].label
-        return loader.render_to_string('form.html', {'self': self}, request=self.request)
+        return loader.render_to_string('form.html', {'component': self}, request=self.request)
 
 
 class ModelForm(Form, django_forms.ModelForm):
@@ -318,6 +342,13 @@ class ModelForm(Form, django_forms.ModelForm):
         if len(self.inner_forms):
             with transaction.atomic():
                 sid = transaction.savepoint()
+
+                # one-to-one field was unchecked
+                for fieldset in self.configured_fieldsets:
+                    for field, form, required, save in fieldset.get('one_to_one', ()):
+                        if not save:
+                            setattr(self.instance, form.prefix, None)
+
                 instance = super(ModelForm, self).save(*args, **kwargs)
                 instance._post_save_form = self
                 try:
@@ -349,6 +380,7 @@ class ModelForm(Form, django_forms.ModelForm):
                         if form.instance.pk:
                             form.instance.delete()
 
+
 class ModelFormOptions(object):
     def __init__(self, options=None):
         self.model = getattr(options, 'model', None)
