@@ -2,6 +2,8 @@
 
 # DJANGO FIELDS
 import sys
+import inspect
+from django.apps import apps
 from django.utils.text import camel_case_to_spaces
 from django.utils.safestring import mark_safe
 
@@ -51,6 +53,14 @@ def list_field_names(model, include_m2m=False):
     return l
 
 
+def list_one_to_one_field_names(model):
+    l = []
+    for field_name in list_field_names(model):
+        if is_one_to_one(model, field_name):
+            l.append(field_name)
+    return l
+
+
 def get_field(model, lookup):
     tokens = lookup.split('__')
     tokens.reverse()
@@ -145,10 +155,19 @@ def set_metadata(cls, attr, value):
     cls._metadata['{}:{}'.format(cls.__name__, attr)] = value
 
 
-def get_metadata(model, attr, default=None, iterable=False):
+def get_metadata(model_or_func, attr, default=None, iterable=False):
+    
     _metadata = None
+    
+    if hasattr(model_or_func, '_metadata'):
+        _metadata = getattr(model_or_func, '_metadata')
+        return _metadata.get('{}:{}'.format(model_or_func.__name__, attr))
+    else:
+        model = model_or_func
+    
+    metaclass = getattr(model_or_func, '_meta')
 
-    if not hasattr(model._meta, attr):
+    if not hasattr(metaclass, attr):
         field_attr = None
 
         if attr == 'search_fields':
@@ -161,7 +180,7 @@ def get_metadata(model, attr, default=None, iterable=False):
         if field_attr:
             _metadata = []
             fields = []
-            for field in model._meta.fields:
+            for field in metaclass.fields:
                 fields.append(field)
             for m2m in list_m2m_fields_with_model(model):
                 fields.append(m2m[0])
@@ -177,9 +196,9 @@ def get_metadata(model, attr, default=None, iterable=False):
                                     _metadata.append(field.name)
                                 else:
                                     _metadata.append('{}__{}'.format(field.name, lookup))
-            setattr(model._meta, field_attr, _metadata)
+            setattr(metaclass, field_attr, _metadata)
     else:
-        _metadata = getattr(model._meta, attr)
+        _metadata = getattr(metaclass, attr)
         if callable(_metadata):
             _metadata = _metadata()
 
@@ -187,9 +206,9 @@ def get_metadata(model, attr, default=None, iterable=False):
         if attr == 'list_display':
             _metadata = []
             fields = []
-            for field in model._meta.fields:
+            for field in metaclass.fields:
                 fields.append(field)
-            for field in model._meta.local_many_to_many:
+            for field in metaclass.local_many_to_many:
                 fields.append(field)
             for field in fields[1:6]:
                 if not field.name.endswith('_ptr') and not field.name == 'ascii' and not field.name == 'tree_index':
@@ -296,7 +315,7 @@ def check_role(self, saving=True):
                 else:
                     Role.objects.get_or_create(user=user, group=group)
                 if role_notify and not already_in_group:
-                    user.send_access_invitation(group)
+                    user.send_access_invitation_for_group(group)
             else:
                 user = User.objects.filter(username=username).first()
                 if user:
@@ -312,7 +331,7 @@ def check_role(self, saving=True):
 
 
 def iterable(string_or_iterable):
-    if string_or_iterable and not type(string_or_iterable) in (list, tuple):
+    if string_or_iterable is not None and not type(string_or_iterable) in (list, tuple):
         return string_or_iterable,
     return string_or_iterable
 
@@ -340,18 +359,23 @@ def getattr_rec(obj, args):
             model = type(obj)
             attr_name = args[0]
             attr = getattr(model, attr_name)
-            value = getattr(obj, attr_name)
+            value = None
             field = None
             if hasattr(attr, 'field_name'):
                 field = getattr(model, '_meta').get_field(attr.field_name)
             elif hasattr(attr, 'field'):
                 field = attr.field
+                if not obj.pk and is_many_to_many(model, attr_name):
+                    value = field.related_model.objects.none()
+
+            if value is None:
+                value = getattr(obj, attr_name)
 
             if field:
                 if hasattr(field, 'formatter') and field.formatter:
-                    from djangoplus.cache import loader
-                    func = loader.formatters[field.formatter]
-                    if len(func.__code__.co_varnames) == 1:
+                    from djangoplus.cache import CACHE
+                    func = CACHE['FORMATTERS'][field.formatter]
+                    if count_parameters_names(func) == 1:
                         value = func(value)
                     else:
                         value = func(value, request=obj.request, verbose_name=field.verbose_name, instance=obj)
@@ -373,18 +397,7 @@ def getattr_rec(obj, args):
                 # it is a method decorated with @meta
                 _metadata = hasattr(value, '_metadata') and getattr(value, '_metadata') or None
                 if _metadata:
-                    verbose_name = _metadata.get('{}:verbose_name'.format(args[0]))
-                    if verbose_name:
-                        value = value()
-                        formatter = _metadata.get('{}:formatter'.format(args[0]))
-                        if formatter:
-                            from djangoplus.cache import loader
-                            func = loader.formatters[formatter]
-                            if len(func.__code__.co_varnames) == 1:
-                                value = func(value)
-                            else:
-                                value = func(value, request=obj.request, verbose_name=verbose_name, instance=obj)
-                            return mark_safe(str(value))
+                    return execute_and_format(obj.request, value)
 
             if callable(value):
                 if type(value).__name__ in ('ManyRelatedManager' or 'RelatedManager'):
@@ -410,30 +423,53 @@ def get_can_execute(action):
     return can_execute
 
 
+def should_add_action(action_inline, action_subsets, current_subset):
+    subsets = []
+    if current_subset == 'all':
+        current_subset = None
+    if action_subsets:
+        for subset_name in action_subsets:
+            if subset_name == 'all':
+                subsets.append(None)
+            else:
+                subsets.append(subset_name)
+        if action_inline:
+            if None not in subsets:
+                subsets.append(None)
+    else:
+        if action_inline:
+            subsets.append(None)
+    return current_subset in subsets
+
+
 def should_filter_or_display(request, model, to):
     if get_metadata(to, 'role_username'):
         can_view = list(
             get_metadata(model, 'can_admin', (), iterable=True) +
             get_metadata(model, 'can_admin_by_organization', (), iterable=True) +
-            get_metadata(model, 'can_admin_by_unit',(), iterable=True) +
+            get_metadata(model, 'can_admin_by_unit', (), iterable=True) +
+            get_metadata(model, 'can_admin_by_role', (), iterable=True) +
             get_metadata(model, 'can_list', (), iterable=True) +
             get_metadata(model, 'can_list_by_organization', (), iterable=True) +
             get_metadata(model, 'can_list_by_unit', (), iterable=True) +
+            get_metadata(model, 'can_list_by_role', (), iterable=True) +
             get_metadata(model, 'can_view', (), iterable=True) +
             get_metadata(model, 'can_view_by_organization', (), iterable=True) +
-            get_metadata(model, 'can_view_by_unit', (), iterable=True))
+            get_metadata(model, 'can_view_by_unit', (), iterable=True) +
+            get_metadata(model, 'can_view_by_role', (), iterable=True))
         if not request.user.is_superuser and can_view and not request.user.in_group(*can_view):
             return False
     return True
 
 
 def find_action(model, action_name):
-    from djangoplus.cache import loader
-    for actions in (loader.actions, loader.class_actions):
+    from djangoplus.cache import CACHE
+    for actions in (CACHE['INSTANCE_ACTIONS'], CACHE['QUERYSET_ACTIONS']):
         for action_group in actions[model]:
             for func_name, action in list(actions[model][action_group].items()):
-                if action['title'] == action_name:
+                if action['verbose_name'] == action_name:
                     return action
+
     return None
 
 
@@ -442,7 +478,7 @@ def find_model_by_verbose_name(verbose_name):
     for model in apps.get_models():
         app_label = get_metadata(model, 'app_label')
         if not app_label.startswith('admin'):
-            if get_metadata(model, 'verbose_name') == verbose_name:
+            if get_metadata(model, 'verbose_name') == verbose_name.replace(' __ ', ' em '):
                 return model
     return None
 
@@ -458,9 +494,9 @@ def find_model_by_verbose_name_plural(verbose_name_plural):
 
 
 def find_subset_by_title(title, model):
-    from djangoplus.cache import loader
-    for subset in loader.subsets[model]:
-        if subset['title'] == title:
+    from djangoplus.cache import CACHE
+    for subset in CACHE['SUBSETS'][model]:
+        if subset['verbose_name'] == title:
             return subset
     return None
 
@@ -482,26 +518,156 @@ def find_model(model, key):
 def find_model_recursively(cls, tokens):
     if tokens:
         token = tokens.pop()
-        attr = getattr(cls, token)
-        if not hasattr(attr, 'field'):
-            return cls
-        return find_model_recursively(attr.field.remote_field.model, tokens)
+        try:
+            attr = getattr(cls, token)
+            if not hasattr(attr, 'field'):
+                return cls
+            return find_model_recursively(attr.field.remote_field.model, tokens)
+        except AttributeError as e:
+            for related_object in get_metadata(cls, 'related_objects'):
+                if related_object.name == token:
+                    return find_model_recursively(related_object.related_model, tokens)
+            raise e
     else:
         return cls
 
 
-def check_condition(condition, obj):
+# function utilities
+
+def check_condition(user, condition, obj):
     satisfied = True
     if obj.pk and condition:
+        model = type(obj)
         attr_name = condition.replace('not ', '')
-        attr = getattr(obj, attr_name)
-        if callable(attr):
-            satisfied = attr()
+        # the method is defined in the model
+        if hasattr(obj, attr_name):
+            attr = getattr(obj, attr_name)
+            if callable(attr):
+                satisfied = attr(*get_role_values_for_condition(attr, user))
+            else:
+                satisfied = bool(attr)
         else:
-            satisfied = bool(attr)
+            # the method is defined in the manager as a subset
+            qs = model.objects.all()
+            if hasattr(qs, attr_name):
+                attr = getattr(qs, attr_name)
+                satisfied = attr().filter(pk=obj.pk).exists()
+            else:
+                raise Exception('The condition "{}" is invalid for {} because it is not an attribute or a method of {},'
+                                ' neither a method of its manager'.format(condition, obj, model))
         if 'not ' in condition:
             satisfied = not satisfied
     return satisfied
 
 
+def get_parameters_names(func, include_annotated=False):
+    parameters_names = []
+    for name, parameter in inspect.signature(func).parameters.items():
+        is_annotated = parameter.annotation is not None and parameter.annotation != inspect._empty
+        if not is_annotated or include_annotated:
+            parameters_names.append(name)
+    return parameters_names
 
+
+def count_parameters_names(func):
+    return len(get_parameters_names(func))
+
+
+def get_role_values_for_condition(func, user):
+    parameters_values = []
+    for name, parameter in inspect.signature(func).parameters.items():
+        if parameter.annotation is not inspect.Parameter.empty:
+            role_username = get_metadata(parameter.annotation, 'role_username')
+            lookup = {role_username: user.username}
+            role = parameter.annotation.objects.filter(**lookup).first()
+            parameters_values.append(role)
+    return parameters_values
+
+
+def get_role_value_for_action(func, user, param_name):
+    for name, parameter in inspect.signature(func).parameters.items():
+        if name == param_name:
+            role_username = get_metadata(parameter.annotation, 'role_username')
+            lookup = {role_username: user.username}
+            return parameter.annotation.objects.filter(**lookup).first()
+    return None
+
+
+def apply_formatter(value, formatter_name, **kwargs):
+    from djangoplus.cache import CACHE
+    formatter = CACHE['FORMATTERS'][formatter_name]
+    if hasattr(formatter, '_formatter'):
+        # it is a function
+        extra_parameter_names = get_parameters_names(formatter)[1:]
+    else:
+        # it is a component
+        extra_parameter_names = get_parameters_names(formatter.__init__)[2:]
+    if extra_parameter_names:
+        extra_parameter_values = {}
+        for parameter_name in extra_parameter_names:
+            if parameter_name in kwargs:
+                extra_paramater_value = kwargs.get(parameter_name, None)
+                extra_parameter_values[parameter_name] = extra_paramater_value
+        f_return = formatter(value, **extra_parameter_values)
+    else:
+        f_return = formatter(value)
+    return mark_safe(f_return)
+
+
+# execute method decorated with @meta
+def execute_and_format(request, func):
+    verbose_name = get_metadata(func, 'verbose_name')
+    formatter_name = get_metadata(func, 'formatter')
+    icon = get_metadata(func, 'icon')
+    params = request and get_role_values_for_condition(func, request.user) or []
+    f_return = func(*params)
+
+    if type(f_return).__name__ == 'QueryStatistics' and not formatter_name:
+        formatter_name = 'statistics'
+    if formatter_name:
+        f_return = apply_formatter(f_return, formatter_name, request=request, title=verbose_name, icon=icon)
+
+    return f_return
+
+
+def get_parameters_details(model, func, input_name):
+    input_params = []
+    if func:
+        param_provider = model
+        if input_name:
+            if '.' in input_name:
+                param_provider = apps.get_model(input_name)
+            else:
+                module_name = model.__module__.replace('models', 'forms')
+                forms_module = __import__(module_name, fromlist=module_name.split('.'))
+                param_provider = getattr(forms_module, input_name)
+        for param_name in get_parameters_names(func, include_annotated=True):
+            if hasattr(param_provider, 'pk'):
+                field = get_field(model, param_name)
+                verbose_name = field.verbose_name
+                param_type = type(field)
+                required = not field.blank
+            else:
+                field = param_provider.base_fields[param_name]
+                verbose_name = field.label
+                param_type = type(field)
+                required = field.required
+            help_text = field.help_text
+            param_type = param_type.__name__.replace('Field', '')
+            param = dict(verbose_name=verbose_name, name=param_name, type=param_type, required=required, help_text=help_text)
+            input_params.append(param)
+    return input_params
+
+
+def get_register_parameters_details(model):
+    input_params = []
+    for field in get_metadata(model, 'fields'):
+        if 'ptr' not in field.name and field.name != 'id':
+            if not hasattr(field, 'exclude') or not field.exclude:
+                param = dict(
+                    verbose_name=field.verbose_name, name=field.name,
+                    type=type(field).__name__.replace('Field', ''),
+                    required=not field.blank, help_text=field.help_text
+                )
+                input_params.append(param)
+    return input_params
